package common;

import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;


public class Logger {
	//private static final String PADDING="                                          ";
	public static final Integer ERROR=new Integer(4);
	public static final Integer WARN=new Integer(3);
	public static final Integer INFO=new Integer(2);
	public static final Integer DEBUG=new Integer(1);
	public static final Integer TRACE=new Integer(0);
	
	public static final String LEVEL_NAME[]={"TRACE","DEBUG","INFO","WARN","ERROR"};
	
	private static ArrayList<LoggerAppenderI> listeners=new ArrayList<LoggerAppenderI>();
	private static Map<String, Logger> loggers=new HashMap<String, Logger>();
	private static SimpleDateFormat dateFormat=new SimpleDateFormat("MM/dd/yyyy");
	private static SimpleDateFormat timeFormat=new SimpleDateFormat("HH:mm:ss.SSS");
	//private static SimpleDateFormat timeFormat=new SimpleDateFormat("HH:mm:ss");
	
	private static int globalLevel;
	
	private int level;
	private String name;
	
	static {
		final String logFileName=PropertiesReader.getInstance().getProperty("util.Logger.file",true);
		
		if (logFileName==null) {
			final LoggerAppenderI defaultAppender=new LoggerAppenderI() {
				public void append(String date, String time, String loggerName, Integer level, String message) {
					String levelName=getLevelName(level);
					if (levelName.length()==4) {
						levelName +=" ";
					}
					String logMessage=date + ' ' + time + ' ' + levelName + " [" + loggerName + "] " + message;
					if (level==ERROR) {
						System.err.println(logMessage);
					} else {
						System.out.println(logMessage);
					}
				}
			};
			addAppender(defaultAppender);
		} else {
			try {
				final PrintWriter fileWriter=new PrintWriter(new FileWriter(logFileName));
				final LoggerAppenderI defaultAppender=new LoggerAppenderI() {
					public void append(String date, String time, String loggerName, Integer level, String message) {
						String levelName=getLevelName(level);
						if (levelName.length()==4) {
							levelName +=" ";
						}
						fileWriter.println(date + ' ' + time + ' ' + levelName + " [" + loggerName + "] " + message);
						fileWriter.flush();
					}
				};
				addAppender(defaultAppender);
			} catch (IOException e) {
				System.out.println("Could not create log file: " + logFileName);
				System.exit(1);
			}
		}
		
		globalLevel=getLoggerLevel(PropertiesReader.getInstance().getProperty("util.Logger.logLevel"));
	}
	
    // Creation & retrieval methods:
    public static Logger getLogger(String name) {
    	Logger logger=(Logger)loggers.get(name);
    	if (logger==null) {

    		//maxLoggerNameLength=Math.max(maxLoggerNameLength, name.length());
    		loggers.put(name, logger=new Logger(name));
  
    		String levelName=PropertiesReader.getInstance().getProperty(name+".logLevel",true);
    		int level=(levelName==null) ? Logger.globalLevel :getLoggerLevel(levelName); 
    		//Breaking this up prevents a circular reference
    		logger.setLevel(level);
    	}
    	return logger;
    }
    // Creation & retrieval methods:
    public static Logger getLogger(Class c) {
    	return getLogger(c.getName());
    }
    
    public static String getLevelName(Integer level) {
    	return LEVEL_NAME[level.intValue()];
    }
    
    public static void addAppender(LoggerAppenderI appender) {
    	listeners.add(appender);
    }

    private static void sendListeners(Integer level, Object message, String loggerName) {
    	Date now=new Date();
    	String date=dateFormat.format(now);
    	String time=timeFormat.format(now);
    	//String paddedLoggerName=loggerName + PADDING.substring(loggerName.length(), maxLoggerNameLength);
    	for (int i=0;i<listeners.size();i++) {
    		LoggerAppenderI listener=(LoggerAppenderI)listeners.get(i);
    		listener.append(date, time, loggerName, level, message.toString());
    	}
    }
    private static int getLoggerLevel(String levelName) {
    	for (int i=0;i<LEVEL_NAME.length;i++) {
    		if (LEVEL_NAME[i].equals(levelName)) {
    			return i;
    		}
    	}
    	return INFO.intValue();
    }
    public Logger(String name) {
    	this.name=name;
    }
    
    // printing methods:
    public final void debug(Object message) {
    	log(DEBUG, message);
    }
    public final void info(Object message) {
    	log(INFO, message);
    }
    public final void warn(Object message) {
    	log(WARN, message);
    }
    public final void error(Object message) {
    	log(ERROR, message);
    }
    public final void trace(Object message) {
    	log(TRACE, message);
    }
    public final boolean isTrace() {
    	return checkLevel(TRACE.intValue());
    }
    public final boolean isDebug() {
    	return checkLevel(DEBUG.intValue());
    }
    public void log(Integer level, Object message) {
    	if (checkLevel(level.intValue())) {
    		Logger.sendListeners(level, message, this.name);
    	}
    }
    private boolean checkLevel(int level) {
    	return level >= this.level;
    }
	public void setLevel(int level) {
		this.level = level;
	}
}