package common.mpeg;

import java.nio.ByteBuffer;

public interface MpegConstants {
	public static final byte B0=(byte)0;
	public static final byte B1=(byte)1;
	public static final byte VIDEO_PACKET=(byte)0xE0;
	public static final byte AUDIO_PACKET=(byte)0xC0;
	public static final byte AUDIO_PACKET_LATM=(byte)0xC1;
	public static final byte PACK_HEADER_PACKET=(byte)0xBA;
	public static final byte SYSTEM_HEADER_PACKET=(byte)0xBB;
	public static final byte PADDING_PACKET=(byte)0xBE;
	public static final int PID_START_FLAG=0x4000;//Elementry Stream & Start
	public static final int TRANSPORT_ERROR_INDICATOR=0x8000;
	public static final int PAYLOAD_UNIT_START_INDICATOR=0x40;
	
	public final byte PRIVATE_PACKET=(byte)0xBD; //Technically this is private stream
	public final byte VC1_PACKET=(byte)0xFD; //
	
	public static final String PROFILE_NAMES[]={"???","HP","Spt","SNR","MP","SP"};
	public static final String LEVEL_NAMES[]={"???","???","???","???","HL","???","H-14","???","ML","???","LL"};
	
	//Part 2 Start Codes (Video)
	public static final byte PICTURE_PACKET=(byte)0x00;
	public static final byte SEQUENCE_PACKET=(byte)0xB3; //-77
	public static final byte SEQUENCE_EXTENSION_PACKET=(byte)0xB5;//-75
	public static final byte SEQUENCE_END_PACKET=(byte)0xB7;
	public static final byte GOP_PACKET=(byte)0xB8;//-72
	
	//The largest possilbe PTS - 1 second
	public static final long LARGEST_PTS=8589934592l-1; //2^33 -1 ----8589932743
	public static final byte PADDING=(byte)0xFF;

	public static final long MHZ=1000000;
	public static final long SYS_CLOCK_FREQ=27*MHZ;		/* 27 MHz clock */
	public static final long SCR_REMAINDER=300;
	public static final long PTS_FREQ= (SYS_CLOCK_FREQ/SCR_REMAINDER);	/* # ticks per second (90kHz) */
	public static final int ASPECT_RATIO[][]={{0,0}, {1,1}, {4,3}, {16,9}, {221,100}};
	
	public static final int MUX_RATE=20000;
	
	public byte EMPTY_PES_HEADER[] = {0x00, 0x00, 0x01, 0x00, 0x00, 0x00, (byte)0x81, 0x00, 0x00};
	public byte PTS_PES_HEADER[] = {0x00, 0x00, 0x01, 0x00, 0x00, 0x00, (byte)0x85, (byte)0x80, 0x05, 0x20, 0x00, 0x00, 0x00, 0x00};
		
	public byte PRIVATE_SUBSTREAM_HEADER[] = {(byte)0x80, 0x01, 0x00, 0x01}; //subStream Id, number of headers, header offset (1 byte)
	public ByteBuffer PACK_HEADER = ByteBuffer.wrap(new byte[]{0x00, 0x00, 0x01, PACK_HEADER_PACKET, 0x44, 0x00, 0x04, 0x00, 0x04, 0x01, 0x00, (byte)0xEA, 0x63, (byte)0xF8});
	public byte[] EOF={0x00,0x00,0x01,(byte)0xB9};
	
	public int TS_PACKET_SIZE=188;
	public int M2TS_PACKET_SIZE=TS_PACKET_SIZE+4;
	
	public byte TS_SYNC=0x47;
	
	public byte TS_STREAM_TYPE_MPEG1_VIDEO=0x01;
	public byte TS_STREAM_TYPE_MPEG2_VIDEO=0x02;
	public byte TS_STREAM_TYPE_MPEG1_AUDIO=0x03;
	public byte TS_STREAM_TYPE_MPEG2_AUDIO=0x04;
	public byte TS_STREAM_TYPE_AAC_AUDIO=0x0F;//ADTS
	public byte TS_STREAM_TYPE_LATM_AUDIO=0x11;//LOAS-AAC
	public byte TS_STREAM_TYPE_AVC_VIDEO=0x1B;
	public byte TS_STREAM_TYPE_PCM_AUDIO=(byte)0x80;
	public byte TS_STREAM_TYPE_AC3_AUDIO=(byte)0x81;
	public byte TS_STREAM_TYPE_DTS_AUDIO=(byte)0x82;
	public byte TS_STREAM_TYPE_TRUEHD_AUDIO=(byte)0x83;
	public byte TS_STREAM_TYPE_EAC3_AUDIO=(byte)0x84;
	public byte TS_STREAM_TYPE_DTS_HIRES_AUDIO=(byte)0x85;
	public byte TS_STREAM_TYPE_DTSHD_AUDIO=(byte)0x86;
	public byte TS_STREAM_TYPE_SUBTITLE=(byte)0x90;
	public byte TS_STREAM_TYPE_INTERACTIVE_STREAM=(byte)0x91;
	public byte TS_STREAM_TYPE_DTS_EXPRESS_AUDIO=(byte)0xa2;
	public byte TS_STREAM_TYPE_VC1_VIDEO=(byte)0xEA;
	
	public byte TS_PAT[] = {0x47, 0x40, 0x00, 0x10, 0x00, 0x00, (byte)0xB0, 0x11, 0x00, 0x00, (byte)0xC1, 0x00, 0x00, 0x00, 0x00, (byte)0xE0, 0x1F, 0x00, 0x01, (byte)0xE1, 0x00, 0x23, 0x5A, (byte)0xAB, (byte)0x82};
	public byte TS_PCR[] = {0x47, 0x10, 0x01, 0x20, (byte)0xB7, 0x10, 0x00, 0x00 , 0x00, 0x00, 0x00, 0x00};
	public byte TS_PMT_HEADER[] = {0x47, 0x41, 0x00, 0x10, 0x00, 0x02, (byte)0xB0, (byte)0xFD, 0x00, 0x01, (byte)0xC1, 0x00, 0x00, (byte)0xF0, 0x01, (byte)0xF0, 0x0C, 0x05, 0x04, 0x48, 0x44, 0x4D, 0x56, (byte)0x88, 0x04, 0x0F, (byte)0xFF, (byte)0xFF, (byte)0xFC};
	//public byte TS_PMT_HEADER[]={0x47, 0x41, 0x00, 0x10, 0x00, 0x02, (byte)0xB0, 0x17, 0x00, 0x01, (byte)0xC1, 0x00, 0x00, (byte)0xF0, 0x00, (byte)0xF0, 0x00};
}
