package common.mpeg;

import java.awt.Dimension;
import java.nio.ByteBuffer;

import common.BitStream;
import common.MuxBuffer;
import common.VideoStreamDescriptor;

public class SequenceHelper {
	private int width;
	private int height;
	private int aspectRatioType;
	private int frameRateType;
	private int vbvSize;
	private int progressiveFlag;
	private int profile=-1;
	private int level=-1;
	
	public SequenceHelper(){};
	
	public SequenceHelper(MuxBuffer buffer, int sequencePacketStart) {
		parse(buffer, sequencePacketStart);
	}
	
	public void parse(MuxBuffer muxBuffer, int sequencePacketStart) {
		ByteBuffer buffer=muxBuffer.getByteBuffer();
		BitStream bitStream=new BitStream(buffer);
		bitStream.setPosition(sequencePacketStart+4);
		width=bitStream.getBits(12);
		height=bitStream.getBits(12);
		
		aspectRatioType=bitStream.getBits(4);
		frameRateType=bitStream.getBits(4);
		int bitRateValue=bitStream.getBits(18);
		int markerBit=bitStream.getBit();
		vbvSize=bitStream.getBits(10);
		int constrainedParametersFlag=bitStream.getBit();
		int loadIntraQuantisermatrix=bitStream.getBit();
		if (loadIntraQuantisermatrix==1) {
			buffer.position(buffer.position()+64);
		}
		int loadNonIntraQuantiserMatrix=bitStream.getBit();
		if (loadNonIntraQuantiserMatrix==1) {
			buffer.position(buffer.position()+64);
		}
		bitStream.byteAlign();
		int test=buffer.getInt();
		//Extension
		if (test==0x000001b5) {
			int extensionStartCodeIdentifier=bitStream.getBits(4);
			//Sequence Extension
			if (extensionStartCodeIdentifier==1) {
				int escapeFlag=bitStream.getBit();
				if (escapeFlag==1) {
					//Discard
					bitStream.getBits(7);
				} else {
					profile=bitStream.getBits(3);
					level=bitStream.getBits(4);
				}
				progressiveFlag=bitStream.getBit();
			}
		}
	}

	public int getAspectRatioType() {
		return aspectRatioType;
	}

	public int getFrameRateType() {
		return frameRateType;
	}

	public int getHeight() {
		return height;
	}

	public int getWidth() {
		return width;
	}
	public Dimension getDisplayDimension() {
		int height=getHeight();
		int width=getWidth();
		//double aspectRatio=MpegVideoPacket.getAspectRatio(getAspectRatioType());
		
		if (height==0 || width==0 /*|| aspectRatio==0*/) {
			return null;
		}
		int displayWidth;
		switch (height) {
			case 480:
				displayWidth=640;
				break;
			case 1088:
				displayWidth=1920;
				height=1080;
				break;
			default:
				displayWidth=width;
		}
		return new Dimension(displayWidth, height);
	}
	public static void setVideoDimensions(byte packet[], int videoSequenceStart, int width, int height) {
		videoSequenceStart +=4;
		packet[videoSequenceStart]=(byte)(width >>> 4);
		packet[++videoSequenceStart]=(byte)((width << 4) | (height >>> 8));
		packet[++videoSequenceStart]=(byte)height;
	}
	public void patchDisplayDimensions(byte packet[], int videoSequenceStart) {
		//parse(packet, videoSequenceStart);
		Dimension display=getDisplayDimension();
		if (display.width != getWidth()) {
			setVideoDimensions(packet, videoSequenceStart, display.width, display.height);
		}
	}
	public void buildDescriptor(VideoStreamDescriptor sd) {
		sd.setFrameRateRatio(MpegUtil.getFrameRateRatio(getFrameRateType()));
		sd.setAspectRatio(MpegConstants.ASPECT_RATIO[getAspectRatioType()]);
		sd.setHeight(getHeight());
		sd.setWidth(getWidth());
		sd.setProfileName(MpegConstants.PROFILE_NAMES[profile]);
		sd.setLevelName(MpegConstants.LEVEL_NAMES[level]);
		sd.setInterlaced(1-progressiveFlag);
	}
	public String toString() {
		StringBuffer out=new StringBuffer();
		out.append(" Video Size: ");
		out.append(width);
		out.append("x");
		out.append(height);
		out.append(" Aspect:");
		out.append(MpegVideoPacket.getAspectRatioText(aspectRatioType));
		if (frameRateType < 9) {
			out.append(" fps: ");
			int frameRateRatio[]=MpegUtil.getFrameRateRatio(frameRateType);
			if (frameRateRatio != null) {
				out.append(frameRateRatio[0]);
				if (frameRateRatio[1]!=1) {
					out.append('/');
					out.append(frameRateRatio[1]);
				}
			}
		}

		return out.toString();
	}
}
