package common.packet;

import java.nio.ByteBuffer;

import common.MuxBuffer;
import common.mpeg.MpegConstants;
import common.mpeg.MpegUtil;


public class PacketPart extends MuxBuffer implements ClockReferenceI {
	private static final long PTS_DTS_NA[]={CLOCK_REFERENCE_NA, CLOCK_REFERENCE_NA};
	protected boolean systemClockReference;
	protected boolean avStream;
	
	public PacketPart(int len) {
		super(len);
	}
	
	public PacketPart(ByteBuffer aBuffer) {
		super(aBuffer);
	}
	protected PacketPart(byte buffer[]) {
		super(buffer);
	}
	public boolean isSystemClockReference() {
		return systemClockReference;
	}

	public void setSystemClockReference(boolean systemClockReference) {
		this.systemClockReference = systemClockReference;
	}
	protected long getSystemClockReference() {
		return ClockReferenceI.CLOCK_REFERENCE_NA;
	}
	
	protected void setSystemClockReference(long clockOffset) {};
	
/*	public long getRelativePts(long ptsOffset, int offset) {
		long outPts=getPts(offset) + ptsOffset;
		if (outPts > MpegConstants.LARGEST_PTS) {
			outPts -=MpegConstants.LARGEST_PTS;
		}
		return outPts;
	}*/
/*	public void updatePtsDts(long clockOffset, int offset) {
		long relativePts=getRelativePts(clockOffset, offset);
		byte ptsDtsFlag=buffer[offset+7];
		if ((ptsDtsFlag & MpegUtil.PTS_PRESENT)==MpegUtil.PTS_PRESENT) {
			MpegUtil.setPesTime(buffer, offset+9, relativePts);
			
			if ((ptsDtsFlag & MpegUtil.DTS_PRESENT)==MpegUtil.DTS_PRESENT) {
				relativePts=getRelativePts(MpegUtil.getPESTime(buffer, offset+14), offset);
				MpegUtil.setPesTime(buffer,offset+14, relativePts);
			}
		}
	}*/	
/*	public long getPts(int offset) {
		int ptsPosition=MpegUtil.getPtsPosition(buffer, offset);
		if (ptsPosition > 0) {
			return MpegUtil.getPESTime(buffer, ptsPosition);
		}
		return Packet.CLOCK_REFERENCE_NONE;
	}*/
	
	public String toString() {
		return "Len: " + buffer.limit();
	}
	
	final private long calculateOffset(long time, final long offset) {
		time+=offset;
		if (time > MpegConstants.LARGEST_PTS) {
			time -=MpegConstants.LARGEST_PTS;
		}
		return time;
	}
	
	protected void updatePtsDts(final long pts, final long dts) {
		int offset=getPesHeaderOffset();
		
		MpegUtil.setPesTime(buffer, offset+9, pts);
		if (dts>=0) {
			MpegUtil.setPesTime(buffer,offset+14, dts);
		}
	}
	
	public void offsetClock(final long clockOffset) {
		setReadWrite();
		long scr=getSystemClockReference();
		if (scr >=0) {
			setSystemClockReference(calculateOffset(scr,clockOffset));
		}
		long ptsDts[]=getPtsDts();
		if (ptsDts[0]>=0) {
			final long pts=calculateOffset(ptsDts[0],clockOffset);
			final long dts=ptsDts[1]>=0?calculateOffset(ptsDts[1],clockOffset):ptsDts[1];
			updatePtsDts(pts,dts);
		}
	}
/*	protected long getPts() {
		if (isAvStream() && hasPesHeader()) {
			int offset=getPesHeaderOffset();
			byte ptsDtsFlag=buffer[offset+7];
			if ((ptsDtsFlag & MpegUtil.PTS_PRESENT)==MpegUtil.PTS_PRESENT) {
				return MpegUtil.getPESTime(buffer, offset+9);
			} else {
				return CLOCK_REFERENCE_NONE;
			}
		}
		return CLOCK_REFERENCE_NA;
	}
*/
	protected long[] getPtsDts() {
		long ptsDts[]=PTS_DTS_NA;
		
		if (isAvStream() && hasPesHeader()) {
			int offset=getPesHeaderOffset();
			byte ptsDtsFlag=buffer.get(offset+7);
			if ((ptsDtsFlag & MpegUtil.PTS_PRESENT)==MpegUtil.PTS_PRESENT) {
				ptsDts[0]=MpegUtil.getPESTime(buffer, offset+9);
				if ((ptsDtsFlag & MpegUtil.DTS_PRESENT)==MpegUtil.DTS_PRESENT) {
					ptsDts[1]=MpegUtil.getPESTime(buffer, offset+14);
				} else {
					ptsDts[1]=CLOCK_REFERENCE_NONE;
				}
			} else {
				ptsDts[0]=ptsDts[1]=CLOCK_REFERENCE_NONE;
			}
		}
		return ptsDts;
	}
	
	
	public long getClockReference(int type) {
		switch (type) {
			case CLOCK_REFERENCE_TYPE_SYSTEM:
				return getSystemClockReference();
			case CLOCK_REFERENCE_TYPE_PTS:
			case CLOCK_REFERENCE_TYPE_DTS:
				if (isAvStream() && hasPesHeader()) {
					int offset=getPesHeaderOffset();
					byte ptsDtsFlag=buffer.get(offset+7);
					if (type==CLOCK_REFERENCE_TYPE_PTS && (ptsDtsFlag & MpegUtil.PTS_PRESENT)==MpegUtil.PTS_PRESENT) {
						return MpegUtil.getPESTime(buffer, offset+9);
					} else if (type==CLOCK_REFERENCE_TYPE_DTS && (ptsDtsFlag & MpegUtil.DTS_PRESENT)==MpegUtil.DTS_PRESENT) {
						return MpegUtil.getPESTime(buffer, offset+14);
					} else {
						return CLOCK_REFERENCE_NONE;
					}
				} else {
					return CLOCK_REFERENCE_NA;
				}
			default:
				return CLOCK_REFERENCE_NA;
		}
	}
	public boolean isAvStream() {
		return avStream;
	}
	public boolean hasPesHeader() {
		int offset=getPesHeaderOffset();
		return (buffer.get(offset)==0 && buffer.get(offset+1)==0 && buffer.get(offset+2)==1);
	}
	public void setAvStream(boolean avStream) {
		this.avStream = avStream;
	}
	public int getPesHeaderOffset() {
		return 0;
	}	
}
