package common.vc1;

import java.io.IOException;
import java.nio.ByteBuffer;

import common.BitStream;
import common.MuxBuffer;

public class Vc1Deinterlacer implements Vc1Constants {
	
	private int frameCounterFlag=0;
	
	public final void copyEntryPoint(int start, int end, ByteBuffer byteBuffer, MuxBuffer buffer) throws IOException {
		buffer.appendFrom(byteBuffer, start, end-start);
	}
	public final void copySequence(int start, int end, ByteBuffer buffer, BitStream bitStream) throws IOException {
		//final int len=end-start;
		//BitStream bitStream=new BitStream(len);
		int index=bitStream.limit()+4;
		bitStream.appendFrom(buffer, start, end-start);
		
		bitStream.setPosition(index);
		int profile = bitStream.getBits(2);
		if (profile!=3) {
			throw new RuntimeException("Only works for Advanced Profile Streams");
		}
		/*int level = bitStream.getBits(3);
		int chromaFormat=bitStream.getBits(2);
		int crap=bitStream.getBits(8);
		int postProcessingFlag=bitStream.getBit();
		int width=(bitStream.getBits(12) + 1)*2;
		int height=(bitStream.getBits(12) + 1)*2;
		int pulldownFlag=bitStream.getBit();*/
		bitStream.skipBits(3+2+8+1+12+12+1);
		int interlacedFlag=bitStream.getBit();
		
		if (interlacedFlag==1) {
			bitStream.skipBits(-1);
			bitStream.setBits(1,0);
		} else {
			throw new RuntimeException("Only works for interlaced streams");
		}
		frameCounterFlag=bitStream.getBit();
		//System.out.println("Pulldown: "+pulldownFlag + " Interlace: " + interlacedFlag + " FrameCounters: " + frameCounterFlag);
		//Common
		int frameInterpolationFlag=bitStream.getBit();
		//Advanced
		int reserved=bitStream.getBit();
		int psfFlag=bitStream.getBit();
		//System.out.println("Interlaced: " + interlacedFlag + " Pulldown: " + pulldownFlag + " PsF: " + psfFlag);
		int displayExtensionFlag=bitStream.getBit();
		if (displayExtensionFlag==1) {
			int displayWidth=bitStream.getBits(14);
			int displayHeight=bitStream.getBits(14);
			int aspectRatioFlag=bitStream.getBit();
			int aspectRatioId=bitStream.getBits(4);
			int aspectRatioWidth, aspectRatioHeight;
			if (aspectRatioId==15) {
				aspectRatioWidth=bitStream.getBits(8);
				aspectRatioHeight=bitStream.getBits(8);
			} else {
				aspectRatioWidth=ASPECT_RATIO[aspectRatioId][0];
				aspectRatioHeight=ASPECT_RATIO[aspectRatioId][1];
			}
			int frameRateFlag=bitStream.getBit();
			if (frameRateFlag==1) {
				int explicitFrameRateFlag=bitStream.getBit();
				int frameRateNumerator, frameRateDenominator;
				
				if (explicitFrameRateFlag==1) {
					frameRateNumerator=bitStream.getBits(16);
					frameRateDenominator=32;
					throw new RuntimeException("Frame rate not 30000/1001");
				} else {
					int frameRateNumeratorId=bitStream.getBits(8);
					frameRateNumerator=FRAME_RATE_NUMERATORS[frameRateNumeratorId];
					if (frameRateNumeratorId == 3) {
						bitStream.skipBits(-8);
						bitStream.setBits(8, 1);
					} else {
						throw new RuntimeException("Frame rate not 30000/1001");
					}
					int frameRateDenominatorId=bitStream.getBits(4);
					frameRateDenominator=FRAME_RATE_DENOMINATORS[frameRateDenominatorId];
				}
			}
		}
		bitStream.setPosition(bitStream.limit());
	}

	public final void copyPicture(int end, BitStream inBitStream, BitStream bitStream) throws IOException {
		ByteBuffer inBuffer=inBitStream.getByteBuffer();
		final int inLimit=inBuffer.limit();
		
		inBuffer.limit(inBuffer.position() + 4);
		bitStream.appendFrom(inBuffer);
		inBuffer.limit(end);
		bitStream.limit(bitStream.capacity());
		int pictureCodingType=inBitStream.getBit();
		if (pictureCodingType==1) {
			throw new IOException("VC1 is not progressive");
		}
		int pType=0;
		//Loop for the picture type until will hit a zero bit or 4 bits
		int bits=0;
		while (bits<4) {
			int bit=inBitStream.getBit();
			pType = (pType << 1) + bit;
			bits++;
			if (bit==0) {
				break;
			}
		}
		bitStream.setBits(bits, pType);
			
		if (frameCounterFlag==1) {
			bitStream.appendBits(8, inBitStream.getByte()&0xff);
		}
		inBitStream.getBits(2); //Get the RPTFRM
		bitStream.appendBits(2, 0);
		//TODO: Deal with Pan and Scan
		bitStream.appendBits(1, inBitStream.getBit()); //Copy RNDCNTRL

		if (pType==Vc1Constants.PTYPE_SKIP) {
			//Should check bitsUsed
			int position=bitStream.getByteBuffer().position() + 1;//
			bitStream.setPosition(position);
			bitStream.limit(position);
		} else {
			inBitStream.getBit();//Remove UVSAMP
			bitStream.appendEscaped(inBitStream, end);
		}
		inBuffer.limit(inLimit);
	}
}
