package ui;

import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.Panel;
import java.awt.TextField;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.util.prefs.Preferences;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

public class FileCutter extends Frame {
	private static final String PREFS_INPUT_DIR="inputDir";
	private Preferences prefs =  Preferences.userRoot().node("RemuxTool");
	private JFileChooser inputFileChooser;
	
	public FileCutter() {
		this.setTitle("File Cutter");
		this.setLayout(new BorderLayout(2,0));
		Panel west=new Panel(new GridLayout(2,1,0,2));
		west.add(new Label("File:"));
		west.add(new Label("Size(MB):"));
		this.add(west, BorderLayout.WEST);

		final TextField fileName=new TextField();
		final TextField sizeMB=new TextField();
		sizeMB.setText("100");
		Panel center=new Panel(new GridLayout(2,1,0,2));
		center.add(fileName);
		Dimension dim=fileName.getPreferredSize();
		dim.width=400;
		fileName.setPreferredSize(dim);
		center.add(sizeMB);
		this.add(center, BorderLayout.CENTER);
		
		Panel right=new Panel(new GridLayout(2,1,0,2));
		Button browse=new Button("Browse...");
		Button cut=new Button("Cut");
		right.add(browse);
		right.add(cut);
		this.add(right, BorderLayout.EAST);
		this.pack();
		
		final ActionListener inputActionListener=new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (inputFileChooser==null) {
					String pathName=prefs.get(PREFS_INPUT_DIR, null);
					inputFileChooser=new JFileChooser(pathName);
				}
				int returnVal = inputFileChooser.showOpenDialog(FileCutter.this);
				if (returnVal==JFileChooser.APPROVE_OPTION) {
					fileName.setText(inputFileChooser.getSelectedFile().getAbsolutePath());
				}
			}
		};
		browse.addActionListener(inputActionListener);
		
		final ActionListener cutActionListener=new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				File inFile=new File(fileName.getText());
				File outFile=new File(fileName.getText()+".cut");
				try {
					FileChannel in=new FileInputStream(inFile).getChannel();
					FileChannel out=new FileOutputStream(outFile).getChannel();
					ByteBuffer buffer=in.map(FileChannel.MapMode.READ_ONLY, 0, Integer.parseInt(sizeMB.getText()) * 1024 * 1024);
					out.write(buffer);
					out.close();
					in.close();
					JOptionPane.showMessageDialog(FileCutter.this,
						    "Created file " + outFile.getAbsolutePath(), 
						    "File Cutter", JOptionPane.INFORMATION_MESSAGE);

				} catch (IOException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
			}
		};
		cut.addActionListener(cutActionListener);
		
		
		this.addWindowListener(new WindowAdapter(){
			@Override
			public void windowClosing(WindowEvent e) {
				dispose();
			}
		});
	}
	
	public static void main(String args[]) {
		new FileCutter().setVisible(true);
	}
}
