package util;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import muxer.MuxManager;
import ui.tonmt.ToNMTPrefs;

import common.FileStream;
import common.Logger;
import common.StreamDescriptor;

public abstract class StreamConverter {
	public static final String BLURAY_EXTENSION=".M2TS";
	public static final String HDDVD_EXTENSION=".EVO";
	
	public static final int SD_VIDEO=0;
	public static final int SD_AUDIO=1;
	public static final int SD_SUBTITLE=2;

	protected Logger logger=Logger.getLogger(this.getClass());
	protected MuxManager muxManager;
	protected ToNMTPrefs prefs;

	public abstract void convert(FileStream fileStream, String outputFileName, List<StreamDescriptor> list) throws IOException ;
	
	public abstract void abort();
	
	protected List<StreamDescriptor> getSubList(List<StreamDescriptor> list, int type) {
		ArrayList<StreamDescriptor> out=new ArrayList<StreamDescriptor>(list.size());
		for (StreamDescriptor sd : list) {
			if (sd.getClassification()==type) {
				out.add(sd);
			}
		}
		return out;
	}
	public StreamDescriptor[][] scan(FileStream fileStream) throws IOException {
		logger.info("Detecting Streams:");
		Map<Integer,StreamDescriptor> streamMap=muxManager.getStreamDescriptorMap(fileStream);
		TreeMap<Integer,StreamDescriptor> audioStreamMap=new TreeMap<Integer,StreamDescriptor>();
		TreeMap<Integer,StreamDescriptor> subtitleStreamMap=new TreeMap<Integer,StreamDescriptor>();
		
		StreamDescriptor video=null;
		for (Iterator<StreamDescriptor> i=streamMap.values().iterator();i.hasNext();) {
			StreamDescriptor sd=i.next();
			logger.info(sd);
			if (video==null && sd.getClassification()==StreamDescriptor.CLASSIFICATION_VIDEO) {
				video=sd;
			} else if (sd.getClassification()==StreamDescriptor.CLASSIFICATION_AUDIO) {
				//audioStreamMap.put(Integer.valueOf(sd.getStreamId()&0xf), sd);
				audioStreamMap.put(Integer.valueOf(sd.getStreamId()), sd);
			} else if (sd.getClassification()==StreamDescriptor.CLASSIFICATION_SUBTITLE) {
				subtitleStreamMap.put(Integer.valueOf(sd.getStreamId()), sd);
			}
		}
		
		if (audioStreamMap.isEmpty() || video ==null) {
			logger.error("Missing audio or video stream!");
			System.exit(-1);
		}
		StreamDescriptor audioArray[]=new StreamDescriptor[audioStreamMap.size()];
		new ArrayList<StreamDescriptor>(audioStreamMap.values()).toArray(audioArray);
		StreamDescriptor subtitleArray[]=new StreamDescriptor[subtitleStreamMap.size()];
		new ArrayList<StreamDescriptor>(subtitleStreamMap.values()).toArray(subtitleArray);
		return new StreamDescriptor[][]{{video}, audioArray, subtitleArray};
	}
	
	public static boolean isBluRay(File file) {
		return file.getAbsolutePath().toUpperCase().endsWith(BLURAY_EXTENSION);
	}
	public static boolean isHdDvd(File file) {
		return file.getAbsolutePath().toUpperCase().endsWith(HDDVD_EXTENSION);
	}
	
	public void postProcess() {
		String action=prefs.getCompletedAction();
		
		if (ToNMTPrefs.COMPLETED_ACTION_NONE.equals(action)) {
			return;
		} else if (ToNMTPrefs.COMPLETED_ACTION_EXIT.equals(action)) {
			System.exit(0);
		} else if (ToNMTPrefs.COMPLETED_ACTION_PROCESS.equals(action)) {
			String process[]=prefs.getCompletedProcess().split(" ");
			try {
				Runtime.getRuntime().exec(process, null, null);
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

}
