package web;

import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.TimeZone;

import common.Logger;


public class MinimalServletResponse {
	public static final int SC_CONTINUE=100;
	public static final int SC_OK=200;
	public static final int SC_PARTIAL_CONTENT=206;
	public static final int SC_UNAUTHORIZED=401;
	public static final int SC_NOT_FOUND=404;
	public static final int SC_REQUESTED_RANGE_NOT_SATISFIABLE=416;
	public static final int SC_INTERNAL_SERVER_ERROR =500;
	public static final int SC_GATEWAY_TIMEOUT=504;
	private static Map SC_TEXT=new HashMap();
	static {
		SC_TEXT.put(new Integer(SC_CONTINUE),"Continue");
		SC_TEXT.put(new Integer(SC_OK),"OK");
		SC_TEXT.put(new Integer(SC_PARTIAL_CONTENT),"Partial Content");
		SC_TEXT.put(new Integer(SC_NOT_FOUND),"Not found");
		SC_TEXT.put(new Integer(SC_UNAUTHORIZED),"Unauthorized");
		SC_TEXT.put(new Integer(SC_INTERNAL_SERVER_ERROR),"Internal Server Error");
		SC_TEXT.put(new Integer(SC_GATEWAY_TIMEOUT),"Gateway Timeout");
		SC_TEXT.put(new Integer(SC_REQUESTED_RANGE_NOT_SATISFIABLE),"Requested Range Not Satisfiable");
	}
	public static final String HTTP="HTTP/1.1 ";
	//private static final String HTTP="HTTP/1.0 ";
	public static final String CONTENT_TYPE="Content-Type";
	
	public static final String CONTENT_TYPE_TEXT="text/html";
	public static final String CONTENT_TYPE_XML="text/xml; charset=\"utf-8\"";
	public static final String CONTENT_TYPE_IMAGE="image/gif";
	public static final String CONTENT_TYPE_MP3="audio/mpeg";
	public static final String CONTENT_TYPE_IMAGE_JPEG="image/jpeg";
	public static final String CONTENT_TYPE_IMAGE_PNG="image/png";
	public static final String CONTENT_TYPE_MPEG="video/mpeg";
	public static final String CONTENT_TYPE_TY="video/ty";
	public static final String CONTENT_TYPE_MPEG2="video/mpeg-2";
	public static final String CONTENT_TYPE_MPEG_STREAM="video/x-mpeg";
	public static final String CONTENT_TYPE_ASX="video/x-ms-asf";
	public static final String CONTENT_TYPE_WMV="video/x-ms-wmv";
	public static final String CONTENT_TYPE_3GPP2="video/3gpp2";
	public static final String CONTENT_TYPE_3GPP="video/3gpp";
	public static final String CONTENT_TYPE_H263="video/x-h263";
	public static final String CONTENT_TYPE_APPLICATION_XHMTL="application/xhtml+xml";
	public static final String CONTENT_TYPE_AUDIO_MP4="audio/mp4";
	public static final String CONTENT_TYPE_QUICKTIME="video/quicktime";
	public static final String CONTENT_TYPE_AVI="video/avi";
	private static final String SET_COOKIE="Set-Cookie";
	
	public static final String AUTHENTICATE="WWW-Authenticate";
	public static final String AUTHENTICATE_BASIC="Basic";
	
	public static final String PRAGMA="Pragma";
	public static final String PRAGMA_STREAM="no-cache, xResetStrm=1";
	
	public static final String CACHE_CONTROL="Cache-Control";
	public static final String CACHE_CONTROL_NO_CACHE="max-age=0, no-cache";
	
	public static final String CONNECTION="Connection";
	public static final String CONNECTION_CLOSE="close";
	public static final String CONNECTION_KEEP_ALIVE="keep-alive";
	
	public static final String CONTENT_LENGTH="Content-Length";
	public static final String DATE="Date";

	public static final String CONTENT_LANGUAGE="Content-Language";
	public static final String CONTENT_LANGUAGE_ENGLISH="en";
	
	public static final String CONTENT_RANGE="Content-Range";
	
	public static final String SERVER="Server";
	public static final String REFRESH="Refresh";
	
	//private SimpleDateFormat dateFormatter= new SimpleDateFormat("EEE, d MMM yyyy HH:mm:ss Z");
	
	private Logger logger=Logger.getLogger(this.getClass());
	private OutputStream outputStream;
	private OutputStreamWriter out;
	//private MinimalServletRequest request;
	private ArrayList headers=new ArrayList();
	private String status;
	private boolean committed=false;
	private SimpleDateFormat formatter=new SimpleDateFormat("EEE, d MMM yyyy HH:mm:ss z");
	private boolean forceClose=true;
	
	public MinimalServletResponse(OutputStream outputStream, MinimalServletRequest request) {
		this.outputStream=outputStream;
		formatter.setTimeZone(TimeZone.getTimeZone("GMT"));
		out=new OutputStreamWriter(outputStream); 
		//this.request=request;
		setStatus(SC_OK);
		
	}
	
	public void setContentType(String contentType) {
		addHeader(CONTENT_TYPE, contentType);
	}
	public OutputStreamWriter getOutputStream() throws IOException {
		writeHeaders();
		return out;
	}
	public OutputStream getOutputStream2() throws IOException {
		writeHeaders();
		return outputStream;
	}
	public void setStatus(int sc) {
		status=HTTP + sc + ' ' + SC_TEXT.get(new Integer(sc));
	}
	private void writeHeaders() throws IOException {
		if (!committed) {
			logger.trace(status);
			out.write(status + "\r\n");
			if (forceClose) {
				addHeader(CONNECTION, CONNECTION_CLOSE);
			}
			for (Iterator i=headers.iterator();i.hasNext();) {
				String header=(String)i.next();
				out.write(header + "\r\n");
				logger.trace(header);
			}
			out.write("\r\n");
//			out.write('\n');
			out.flush();
			committed=true;
		}
	}
	
	public void addHeader(String name, String value) {
		headers.add(name + ": " + value);
	}
	public void addCookie(Cookie cookie) {
		//Do I need to encode the cookie data???
		addHeader(SET_COOKIE, cookie.getName() + '=' + cookie.getValue() + "; path=" + cookie.getPath());
		//TODO: //Other cookie pieces
	}
	public boolean isCommitted() {
		return committed;
	}
	/**
	 * Set to "true" before get outputStream to not write HTTP header
	 * @param committed
	 */
	public void setCommitted(boolean committed) {
		this.committed=committed;
	}
	/**
	 * Technically, this should be an int, but many HD files are actually > 2GB-1
	 * @param len
	 */
	public void setContentLength(int len) {
		addHeader(CONTENT_LENGTH, len+"");
		forceClose=false;
	}
	/**
	 * @param len
	 */
	public void setContentLength(long len) {
		addHeader(CONTENT_LENGTH, len+"");
		forceClose=false;
	}
	
	public void setDateHeader(String name, long date) {
		addHeader(DATE, formatter.format(new Date(date)));		
	}
	public void flushBuffer() throws IOException {
		if (!committed) {
			setContentLength(0);
		}
		writeHeaders();
		out.flush();
		outputStream.flush();
	}
	public boolean isForceClose() {
		return forceClose;
	}
	public void httpContinue() throws IOException {
		setStatus(SC_CONTINUE);
		out.write(status + "\r\n\r\n");
		out.flush();
		setStatus(SC_OK);
	}
}
