package web;

import java.io.IOException;
import java.net.Socket;

import common.Logger;

public class ServerConnection implements Runnable {
	private Socket socket;
	private WebServer webServer;
	private Logger logger=Logger.getLogger(this.getClass());
	
	public void init(Socket socket, WebServer webServer) {
		this.socket=socket;
		this.webServer=webServer;
	}
	public void run() {
		while (true) {
			MinimalServletRequest req;
			try {
				//socket.setOOBInline(true);
				req=new MinimalServletRequest(socket.getInputStream(), socket.getInetAddress());
				req.parse();
			} catch (IOException ioe) {
				break;
			}
			try {
				MinimalServletResponse resp=new MinimalServletResponse(socket.getOutputStream(), req);
				SecurityInterceptorI securityInterceptor=webServer.getSecurityInterceptor();
				if (securityInterceptor ==null || securityInterceptor.doAuthorization(req, resp)) {
					MinimalServlet minimalServer=webServer.getServlet(req.getPath());
					minimalServer.service(req, resp);
				}
				resp.flushBuffer();
				String connection=req.getHeader(MinimalServletResponse.CONNECTION);
				if (resp.isForceClose() || (connection != null && connection.toLowerCase().equals(MinimalServletResponse.CONNECTION_CLOSE))) {
					logger.debug("Closing socket");
					socket.close();
					break;
				}
			} catch (IOException ioe) {
				logger.info("Uncaught IOException: " + ioe.getMessage());
				//System.out.println("ServerConnection");
				//ioe.printStackTrace();
				try {
					socket.close();
				} catch (IOException e) {
				}
				break;
			}
			/*try {
				socket.close();
				break;
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}*/
		}
	}
}
